#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""Initialize ewccli package."""

__title__ = "ewccli"
__version__ = "0.3.0"
__author__ = "European Weather Cloud <support@europeanweather.cloud>"

__all__ = [
    "__title__",
    "__version__",
]
